package gallery
{
	import flash.display.Loader;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Rectangle;
	
	import preloader.SimpleLinePreloader;
	
	import thumbAreas.SimpleLinearThumbArea;
	import thumbAreas.ThumbAreaSettings;
	import thumbAreas.ThumbEvent;
	
	import tools.BulkLoader;
	import tools.Statics;
	
	import transitions.TransitionManager;
	
	public class SimpleGallery extends Sprite
	{
		public static var itemAreaReady:String = "itemAreaReady"; 
		
		private var itemLoader:Loader;
		private var itemsPath:Array;
		private var itemTransitions:Array;
		private var itemSprite:Sprite;
		private var itemSpriteWidth:Number;
		private var itemSpriteHeight:Number;
		private var currentItem:IGalleryItem;
		private var nextItem:IGalleryItem;
		private var currentItemInd:Number;
		private var loadingItemInd:Number;
		private var itempreLoader:SimpleLinePreloader;
		private var thumbArea:SimpleLinearThumbArea;
		public function SimpleGallery(itemSpriteWidth:Number,itemSpriteHeight:Number,itemsPath:Array,itemTransitions:Array,thumbsPath:Array = null,thumbAreaSettings:ThumbAreaSettings = null)
		{
			this.itemSpriteWidth = itemSpriteWidth;
			this.itemSpriteHeight = itemSpriteHeight;
			this.itemsPath = itemsPath;
			this.itemTransitions = itemTransitions;
			
			TransitionManager.transitionAreaWidth = Statics.itemAreaWidth;
			TransitionManager.transitionAreaHeight = Statics.itemAreaHeight;
			
			itemSprite = new Sprite();
			addChild(itemSprite);

			if(thumbsPath != null){
				if(thumbAreaSettings == null)
					thumbAreaSettings = new ThumbAreaSettings();
				thumbArea = new SimpleLinearThumbArea(thumbAreaSettings,thumbsPath);
				addChild(thumbArea);
				
				thumbArea.addEventListener(ThumbEvent.Thumb_Clicked,thumbClicked);
				var remainingBox:Rectangle = thumbArea.getRemainingBox();
				itemSprite.x = (remainingBox.width - Statics.itemAreaWidth) / 2 + remainingBox.x;
				itemSprite.y = (remainingBox.height - Statics.itemAreaHeight) / 2 + remainingBox.y;
				thumbArea.addEventListener(SimpleLinearThumbArea.Thumbs_Loaded,thumbsLoaded);
			}else{
				thumbArea = null;
				itemSprite.x = (Statics.stageWidth - Statics.itemAreaWidth) / 2;
				itemSprite.y = (Statics.stageHeight - Statics.itemAreaHeight) / 2;
			}
			var itemSpriteMask:Sprite = new Sprite();
			itemSpriteMask.graphics.lineStyle(0, 0, 0);
			itemSpriteMask.graphics.beginFill(0x000000, 1);
			itemSpriteMask.graphics.drawRect(itemSprite.x, itemSprite.y, itemSpriteWidth, itemSpriteHeight);
			itemSpriteMask.graphics.endFill();
			addChild(itemSpriteMask);
			itemSprite.mask = itemSpriteMask;
			
			
			currentItem = new ImageItem("",itemSpriteWidth,itemSpriteHeight);
			itemSprite.addChild(currentItem as Sprite);
			nextItem = new ImageItem("",itemSpriteWidth,itemSpriteHeight);
			
			currentItemInd = -1;
			loadingItemInd = -1;
			
			itempreLoader = new SimpleLinePreloader(0xFF0000,0,0,itemSpriteWidth);
			addChild(itempreLoader);
			
			var shapeURLs:Array = new Array();
			
			for (var i:uint; i < itemTransitions.length; i++) {
				if ( itemTransitions[i] != null && itemTransitions[i].type == TransitionManager.Shapes) {
					if(itemTransitions[i].shapeURL!=""){
						shapeURLs.push(itemTransitions[i].shapeURL)
					}
				}
			}
			if (Statics.transition.type == TransitionManager.Shapes){
				if(Statics.transition.shapeURL!=null){
					shapeURLs.push(Statics.transition.shapeURL)
				}
			}
			
			Statics.shapeLoader = new BulkLoader(shapeURLs);
			Statics.shapeLoader.addEventListener("AllLoaded", shapesLoaded);
			Statics.shapeLoader.load();
		}
		private function shapesLoaded(e:Event):void{
			dispatchEvent(new Event(itemAreaReady));
			if(thumbArea != null){
				thumbArea.load();
			}else{
				thumbsLoaded(null);
			}
			
		}
		private function thumbsLoaded(e:Event):void{
			loadItem(0);
		}
		private function thumbClicked(e:ThumbEvent):void{
			loadItem(e.id);
		}
		public function loadItem(ind:Number):void
		{
			if (ind == currentItemInd || ind == loadingItemInd) {
				return;
			}
			if (nextItem && nextItem.isLoading()){
				nextItem.stopLoading();
			}
			loadingItemInd = ind;
			if(TransitionManager.hasTransition()){
				TransitionManager.removeTransition();	
			}else{
				nextItem = new ImageItem(itemsPath[loadingItemInd],itemSpriteWidth,itemSpriteHeight,itempreLoader);
				Sprite(nextItem).addEventListener("ItemReady",itemLoaded);
				nextItem.load();
			}
		}
		
		private function itemLoaded(e:Event):void
		{
			Sprite(nextItem).removeEventListener("ItemReady",itemLoaded);
			itemSprite.addChild(nextItem as Sprite);
			Sprite(nextItem).visible = false;
			currentItemInd = loadingItemInd;
			loadingItemInd = -1;
			showNewItem();
		}
		private function transitionCompleted():void{
			itemSprite.removeChild(Sprite(currentItem));
			currentItem = nextItem;
			nextItem = null;
			if(loadingItemInd != -1){
				nextItem = new ImageItem(itemsPath[loadingItemInd],itemSpriteWidth,itemSpriteHeight,itempreLoader);
				Sprite(nextItem).addEventListener("ItemReady",itemLoaded);
				nextItem.load();
			}
		}
		private function showNewItem():void {
			Sprite(nextItem).visible = true;
			if ( itemTransitions[currentItemInd] == null){
				TransitionManager.addTransition(Sprite(nextItem), Sprite(currentItem) , Statics.transition,transitionCompleted);
			}else{
				TransitionManager.addTransition(Sprite(nextItem), Sprite(currentItem) , itemTransitions[currentItemInd],transitionCompleted);
			}
		}
	}
}