package transitions
{
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.geom.Rectangle;

	public class ExplodeTransition extends Transition
	{
		private var current:Sprite;
		private var old:Sprite;
		private var velocityX:Array;
		private var velocityY:Array;
		private var gravity:Number;
		private var grids:Array;
		private var bitmapData:BitmapData;
		private var numOut:Number;
		private var gWidth:Number;
		private var gHeight:Number;
		private var oldHeight:Number;
		public function ExplodeTransition(current:Sprite, old:Sprite, rowAmount:Number = 8,colAmount:Number=8,
										  velocityXMin:Number=-5,velocityXMax:Number=5,velocityYMin:Number=-20,
										  velocityYMax:Number=-10,gravity:Number=2) 
		{
			this.current = current;
			this.old = old;
			this.gravity = gravity;
			
			oldHeight = old.parent.height;
			numOut = 0;
			grids = new Array();
			velocityX = new Array();
			velocityY = new Array();
			if(old){
				if(current.parent.getChildIndex(current)>current.parent.getChildIndex(old))
					current.parent.swapChildren(current, old);
				
				
				bitmapData = new BitmapData(TransitionManager.transitionAreaWidth, TransitionManager.transitionAreaHeight,true,0x00000000);
				bitmapData.draw(old);
				
				gWidth = Math.floor(TransitionManager.transitionAreaWidth/colAmount);
				gHeight = Math.floor(TransitionManager.transitionAreaHeight/rowAmount);
				
				var remainingWidth:Number = TransitionManager.transitionAreaWidth - gWidth*colAmount;
				var remainingHeight:Number = TransitionManager.transitionAreaHeight - gHeight*rowAmount;
				
				for(var i:int = 0;i<colAmount;i++){
					for(var j:int = 0;j<rowAmount;j++){
						var w:Number = gWidth;
						var h:Number = gHeight;
						if(i==rowAmount-1){
							h += remainingHeight;
						}
						if(j==colAmount-1){
							w += remainingWidth;
						}
						var bd:BitmapData = new BitmapData(gWidth, gHeight);
						bd.copyPixels(bitmapData, new Rectangle(i*gWidth, j*gHeight, w, h), new Point(0, 0));
						var grid:Bitmap = new Bitmap(bd);
						grid.x = i*gWidth;
						grid.y = j*gHeight;
						grids.push(grid);
						old.parent.addChild(grid);
						velocityX.push(velocityXMin+(velocityXMax-velocityXMin)*Math.random());
						velocityY.push(velocityYMin+(velocityYMax-velocityYMin)*Math.random());
					}
				}
				
				old.visible = false;
				old.parent.addEventListener(Event.ENTER_FRAME, updateGrids);
			
			}
			else{
				completeTransition();
			}
			
			
		}
		
		private function timerTick(e:Event):void{
			completeTransition();
		}
		
		
		private function updateGrids(e:Event):void
		{
			numOut = 0;
			for (var i:int = 0; i < grids.length; i++)
			{
				grids[i].x += velocityX[i];
				grids[i].y += velocityY[i];
				velocityY[i] += gravity;
				if(grids[i].y > oldHeight){
					numOut++;
				}
			}
			
			if(numOut == grids.length){
				completeTransition();
			}
		}
		
		public override function completeTransition():void {
			for (var i:int = 0; i < grids.length; i++){
				old.parent.removeChild(grids[i]);
			}
			bitmapData.dispose();
			old.parent.removeEventListener(Event.ENTER_FRAME, updateGrids);
			dispatchEvent(new TransitionEvent(TransitionEvent.TransitionCompleted, current,old));
		}
	}
}